@@ -15,10 +15,6 @@ def operator(kw: str) -> Reserved:
     return Reserved(kw, TokenType.OPERATOR)
 
 
-def builtin(kw: str) -> Reserved:
-    return Reserved(kw, TokenType.BUILT_IN)
-
-
 def process_boolean(op):
     match op:
         case 'on' | 'true' | 'enable':
@@ -33,6 +29,7 @@ id_tag = Tag(TokenType.ID)
 num = Tag(TokenType.INT) ^ (lambda x: int(x))
 float_num = Tag(TokenType.FLOAT) ^ (lambda x: float(x))
 boolean = Tag(TokenType.BOOL) ^ process_boolean
+null = keyword('null') ^ (lambda x: avo_ast.NullAST())
 string = Tag(TokenType.STRING) ^ (lambda x: avo_ast.StringAST(x[1:-1]))
 a_expr_precedence_levels = [
     ['*', '/'],
@@ -41,9 +38,6 @@ a_expr_precedence_levels = [
 relational_operators = ['==', '!=', '>=', '<=', '<', '>']
 unary_operators = ['--', '++', '-']
 assign_operators = ['+=', '-=', '*=', '/=', '=']
-builtins = [
-    'int', 'float', 'string', 'length', 'range'
-]
 b_expr_precedence_levels = [
     ['and', '&&'],
     ['or', '||'],
@@ -52,13 +46,35 @@ b_expr_precedence_levels = [
 
 
 def array_expr():
+    """Array expression
+    [x, y, z, ...]
+    """
     def process(p):
         (_, data), _ = p
         return avo_ast.ArrayAST(data)
     return keyword('[') + Opt(Rep(Lazy(expr) + Opt(keyword(',')))) + keyword(']') ^ process
 
 
+def array_generator_expr():
+    """Array generator expression
+    [i for i in object if i > x]
+    """
+    def process(p):
+        ((((((_, var), _), val), _), obj), condition), _ = p
+        if condition is not None:
+            _, condition = condition
+        return avo_ast.GeneratorAST(var, val, obj, condition)
+    return (
+            keyword('[') + expression() + keyword('for') + id_tag + operator('in') + expression() +
+            Opt(keyword('if') + Exp(b_expr(), None)) + keyword(']')
+    ) ^ process
+
+
 def if_else_expr():
+    """Ternary operator
+    condition ? x : y
+    x if condition else y
+    """
     def process(p):
         (((body, op1), condition), op2), else_body = p
         return avo_ast.TernaryOpAST(body, op1, condition, op2, else_body)
@@ -69,6 +85,9 @@ def if_else_expr():
 
 
 def lambda_stmt():
+    """Lambda statement
+    (a, b, c) => {...}
+    """
     def process(p):
         (((((_, args), _), _), _), statements), _ = p
         arguments = []
@@ -78,7 +97,6 @@ def lambda_stmt():
             else:
                 arguments.append(avo_ast.ArgumentAST(arg.value[0][0], arg.value[0][1][1]))
         return avo_ast.LambdaStmt(arguments, statements)
-
     return (
             keyword('(') + Rep(id_tag + Opt(operator('=') + Lazy(expression)) + Opt(keyword(','))) +
             keyword(')') + operator('=>') + keyword('{') + Opt(Lazy(stmt_list)) + keyword('}')
@@ -86,6 +104,9 @@ def lambda_stmt():
 
 
 def module_obj_expr():
+    """Module object expression
+    x.y
+    """
     def process(p):
         (module, _), var = p
         return avo_ast.ModuleCallAST(module, var)
@@ -93,17 +114,24 @@ def module_obj_expr():
 
 
 def id_or_module():
-    return class_property_stmt() | module_obj_expr() | id_tag
+    return class_property_expr() | module_obj_expr() | id_tag
 
 
 def a_expr_value():
     return (
-            Lazy(built_in_func_stmt) |
+            Lazy(call_stmt) |
+            Lazy(array_generator_expr) |
+            Lazy(array_expr) |
+            Lazy(read_stmt) |
+            brace_expr() |
+            module_obj_expr() |
+            Lazy(class_property_expr) |
             (num ^ (lambda x: avo_ast.IntAST(x))) |
             (float_num ^ (lambda x: avo_ast.FloatAST(x))) |
             (id_tag ^ (lambda x: avo_ast.VarAST(x))) |
             (boolean ^ (lambda x: avo_ast.BoolAST(x))) |
-            string
+            string |
+            null
     )
 
 
@@ -118,12 +146,6 @@ def a_expr_group():
 
 def a_expr_term():
     return (
-            module_obj_expr() |
-            brace_expr() |
-            Lazy(array_expr) |
-            Lazy(read_stmt) |
-            Lazy(call_stmt) |
-            Lazy(class_property_stmt) |
             a_expr_value() |
             a_expr_group() |
             unary_op_stmt()
@@ -139,11 +161,6 @@ def any_op_in_list(ops):
     return reduce(lambda l, r: l | r, op_parsers)
 
 
-def any_builtin_in_list(blt):
-    parsers = [builtin(i) for i in blt]
-    return reduce(lambda l, r: l | r, parsers)
-
-
 def precedence(val_parser, levels, combine):
     def op_parser(level):
         return any_op_in_list(level) ^ combine
@@ -164,13 +181,23 @@ def process_relop(p):
 
 
 def b_expr_relop():
+    """Relation operation expression
+    x >= y
+    x == y
+    x != y
+    etc
+    """
     return (
             a_expr() + any_op_in_list(relational_operators) + a_expr()
     ) ^ process_relop
 
 
 def b_expr_not():
-    return (keyword('not') + Lazy(b_expr_term)) ^ (lambda p: avo_ast.NotOp(p[1]))
+    """Not expression
+    not x
+    !x
+    """
+    return (Alt(keyword('not'), operator('!')) + Lazy(b_expr_term)) ^ (lambda p: avo_ast.NotOp(p[1]))
 
 
 def b_expr_group():
@@ -198,6 +225,9 @@ def b_expr():
 
 
 def brace_expr():
+    """Brace expression
+    x[y][z]
+    """
     def process(p):
         (((obj, _), v), _), v_arr = p
         arr = []
@@ -218,7 +248,10 @@ def expr():
     )
 
 
-def class_property_stmt():
+def class_property_expr():
+    """Class property statement
+    x::y
+    """
     def process(p):
         ((is_super, name), _), var = p
         if is_super is not None:
@@ -233,13 +266,21 @@ def expression():
 
 # --== statements ==-- #
 def assign_stmt():
+    """Assign statement
+    var x = y
+    """
     def process(p):
         ((_, name), _), e = p
         return avo_ast.AssignStmt(name, e, False, True)
-    return (keyword('var') + id_tag + operator('=') + expression()) ^ process
+    return (
+            keyword('var') + id_tag + operator('=') + expression()
+    ) ^ process
 
 
 def assign_const_stmt():
+    """Assign constant statement
+    const x = y
+    """
     def process(p):
         ((_, name), _), e = p
         return avo_ast.AssignStmt(name, e, True, True)
@@ -247,6 +288,9 @@ def assign_const_stmt():
 
 
 def reassign_stmt():
+    """Reassign statement
+    x = y
+    """
     def process(p):
         (name, op), e = p
         return avo_ast.AssignStmt(name, e, False, False, op)
@@ -254,6 +298,10 @@ def reassign_stmt():
 
 
 def unary_op_stmt():
+    """Unary operator statement
+    x++
+    ++x
+    """
     def process(p):
         sym, name = p
         if sym not in unary_operators:
@@ -276,6 +324,17 @@ def block_stmt():
 
 
 def if_stmt():
+    """if-elif-else statement
+    if condition1 {
+        body1
+    } elif condition2 {
+        body2
+    } elif condition3 {
+        body3
+    } else {
+        body4
+    }
+    """
     def process(p):
         (((((_, condition), _), body), _), elif_array), false_p = p
         if false_p:
@@ -298,6 +357,11 @@ def if_stmt():
 
 
 def while_stmt():
+    """While statement
+    while condition {
+        body
+    }
+    """
     def process(p):
         (((_, condition), _), body), _ = p
         return avo_ast.WhileStmt(condition, body)
@@ -306,14 +370,23 @@ def while_stmt():
 
 
 def break_stmt():
+    """Break statement
+    break
+    """
     return keyword('break') ^ (lambda x: avo_ast.BreakStmt())
 
 
 def continue_stmt():
+    """Continue statement
+    continue
+    """
     return keyword('continue') ^ (lambda x: avo_ast.ContinueStmt())
 
 
 def echo_stmt():
+    """Echo statement
+    echo(x, y, z, ...)
+    """
     def process(p):
         (_, data), _ = p
         return avo_ast.EchoStmt(data)
@@ -328,26 +401,21 @@ def echo_stmt():
 
 
 def read_stmt():
+    """Read statement
+    x = read(...)
+    """
     def process(p):
         _, text = p
         return avo_ast.ReadStmt(text)
     return keyword('read') + expression() ^ process
 
 
-def built_in_func_stmt():
-    def process(p):
-        ((name, _), arg), _ = p
-        args = []
-        for a in arg:
-            args.append(a.value[0])
-        return avo_ast.BuiltInFuncStmt(name, args)
-    return (
-            any_builtin_in_list(builtins) + keyword('(') +
-            Rep(Lazy(expression) + Opt(keyword(','))) + keyword(')')
-    ) ^ process
-
-
 def func_stmt():
+    """Function assign statement
+    func name(args) {
+        body
+    }
+    """
     def process(p):
         ((((((_, func_name), _), args), _), _), statements), _ = p
         arguments = []
@@ -365,6 +433,9 @@ def func_stmt():
 
 
 def interface_func_stmt():
+    """Interface function statement
+    func name()
+    """
     def process(p):
         (((_, func_name), _), args), _ = p
         arguments = []
@@ -382,6 +453,12 @@ def interface_func_stmt():
 
 
 def call_stmt():
+    """Call statement
+    func_name(args)
+    func_name(args) with {
+        lambda body
+    }
+    """
     def process(p):
         (((func_name, _), args), _), l = p
         arguments = []
@@ -396,11 +473,14 @@ def call_stmt():
     return (
         id_or_module() + keyword('(') +
         Rep(Opt(id_tag + operator('=')) + expression() + Opt(keyword(','))) +
-        keyword(')') + Opt(keyword('{') + Opt(Lazy(stmt_list)) + keyword('}'))
+        keyword(')') + Opt(keyword('with') + keyword('{') + Opt(Lazy(stmt_list)) + keyword('}'))
     ) ^ process
 
 
 def return_stmt():
+    """Return statement
+    return x
+    """
     def process(p):
         _, return_value = p
         return avo_ast.ReturnStmt(return_value)
@@ -408,6 +488,11 @@ def return_stmt():
 
 
 def for_stmt():
+    """For statement
+    for var x = y; condition; x++ {
+        body
+    }
+    """
     def process(p):
         (((((((_, var), _), cond), _), action), _), body), _ = p
         return avo_ast.ForStmt(var, cond, action, body)
@@ -419,6 +504,11 @@ def for_stmt():
 
 
 def foreach_stmt():
+    """Foreach statement
+    for i in object {
+        body
+    }
+    """
     def process(p):
         (((((_, var), _), val), _), body), _ = p
         return avo_ast.ForStmt(var, val, body, None)
@@ -429,6 +519,11 @@ def foreach_stmt():
 
 
 def import_stmt():
+    """Import statement
+    import module
+    import a1, a2, a3, ...
+    from module_name import a, b, c, ...
+    """
     def process(p):
         ((x, module), _), objects = p
         objects = [i.value[0] for i in objects]
@@ -445,6 +540,13 @@ def import_stmt():
 
 
 def switch_case_stmt():
+    """Switch-case-else statement
+    switch object {
+        case x {body1}
+        case [y, z, w] {body2}
+        else {body3}
+    }
+    """
     def process(p):
         ((((_, var), _), cases), else_body), _ = p
         cases_list = []
@@ -466,6 +568,11 @@ def switch_case_stmt():
 
 
 def assign_class_stmt():
+    """Class assign statement
+    [abstract] class MyClass {
+        body
+    }
+    """
     def process(p):
         ((((((prefix, _), name), inherit), interfaces), _), body), _ = p
         if inherit:
@@ -484,6 +591,11 @@ def assign_class_stmt():
 
 
 def assign_interface_stmt():
+    """Interface assign statement
+    interface Name {
+        body
+    }
+    """
     def process(p):
         (((_, name), _), body), _ = p
         return avo_ast.InterfaceStmt(name, body)
@@ -493,6 +605,7 @@ def assign_interface_stmt():
 
 
 def class_body():
+    """Class body"""
     def process(p):
         return avo_ast.StmtList(p)
     return Rep(
@@ -501,6 +614,7 @@ def class_body():
 
 
 def interface_body():
+    """Interface body"""
     def process(p):
         return avo_ast.StmtList(p)
     return Rep(
@@ -509,6 +623,11 @@ def interface_body():
 
 
 def init_class_stmt():
+    """Assign class init func
+    init(args) {
+        body
+    }
+    """
     def process(p):
         (((_, args), _), body), _ = p
         arguments = []
@@ -547,6 +666,13 @@ def interface_body_stmt():
 
 
 def try_catch_stmt():
+    """Try-catch statement
+    try {
+        error code
+    } catch e {
+        catch error
+    }
+    """
     def process(p):
         ((((((_, try_body), _), _), e_name), _), catch_body), _ = p
         return avo_ast.TryCatchStmt(try_body, e_name, catch_body)
